"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.rpxtorem = void 0;
const postcss_1 = __importDefault(require("postcss"));
const comment_1 = require("../../utils/comment");
const index_1 = require("../../utils/index");
const option_1 = require("../option");
const DefualtRpxRootValue = 32;
const DefaultRpx2PxRatio = 2;
function default_1(source, options) {
    const plugins = [
        // rpx - rem
        ComponentNameMapPlugin(options),
        ConditionalCompilatioPlugin(options),
        ...(options.plugins || []).map((i) => {
            if (i instanceof Array) {
                return require(i[0])(i.length > 1 ? i[1] : {});
            }
            else {
                return require(i)();
            }
        })
    ];
    const { syntax } = options;
    const config = {};
    if (options.usePx) {
        plugins.push(require('../../webpack/plugins/postcss-rpx2px')({
            ratio: options.usePx.ratio || DefaultRpx2PxRatio
        }));
    }
    else if (options.needRpx !== false) {
        plugins.push(require('../../webpack/plugins/postcss-rpx2rem/index')({
            rootValue: options.rpxRootValue || DefualtRpxRootValue,
            propList: ['*']
        }));
    }
    if (syntax && typeof syntax === 'object') {
        config.syntax = syntax;
    }
    if ((0, index_1.isEnableStyleScope)(options) &&
        options.resourcePath !== `${options.rootPath}/app.acss`) {
        // 样式隔离
        plugins.push(classScopePlugin({
            hash: (0, option_1.randomHash)(options),
            isComponent: options.config && options.config.component
        }));
    }
    return (0, postcss_1.default)(plugins).process(source, config).toString();
}
exports.default = default_1;
function rpxtorem(csstext, options) {
    if (options.usePx) {
        return (0, postcss_1.default)([
            require('../../webpack/plugins/postcss-rpx2px')({
                ratio: options.usePx.ratio || DefaultRpx2PxRatio
            })
        ])
            .process(csstext)
            .toString();
    }
    else {
        return (0, postcss_1.default)([
            require('../../webpack/plugins/postcss-rpx2rem/index')({
                rootValue: options.rpxRootValue || DefualtRpxRootValue,
                propList: ['*']
            })
        ])
            .process(csstext)
            .toString();
    }
}
exports.rpxtorem = rpxtorem;
// 样式隔离插件
const classScopePlugin = postcss_1.default.plugin('class-scope-plugin', function (opts) {
    return function (root) {
        root.nodes.forEach((rule) => {
            if (rule.type === 'rule') {
                rule.selector = rule.selector
                    .split(' ')
                    .map((s) => {
                    return s
                        .split(',')
                        .map((s) => {
                        return addScope(s, opts.hash, opts.isComponent);
                    })
                        .join(',');
                })
                    .join(' ');
            }
        });
    };
});
function addScope(selector, hash, isComponent) {
    // 仅针对非组件做变动处理，因为 tiga-page 上有hash值
    if (!isComponent)
        return `.${hash} ${selector}`;
    const index = selector.indexOf(':');
    if (index >= 0) {
        // 对于 :after :before 这样的样式需要做特殊处理
        return selector.replace(':', `.${hash}:`);
    }
    return `${selector}.${hash}`;
}
const ComponentNameMapPlugin = postcss_1.default.plugin('global-component-name-plugin', function (options) {
    return function (root) {
        root.walkRules((rule) => {
            rule.selector = rule.selector
                .split(' ')
                .map((s) => {
                return s
                    .split(',')
                    .map((s) => {
                    return mapComponentName(s.trim(), options);
                })
                    .join(',');
            })
                .join(' ');
        });
    };
});
function mapComponentName(selector, options) {
    const arr = selector.split(':');
    const name = arr[0];
    const externalComponent = (0, index_1.getExternalComponent)(options, name);
    // 扩展组件不使用自定义tag(tiga-xx)
    if (!externalComponent) {
        const globalName = (0, option_1.globalComponentName)(name, options);
        if (globalName) {
            arr[0] = globalName;
        }
    }
    return arr.join(':');
}
const ConditionalCompilatioPlugin = postcss_1.default.plugin('conditional-compilatio-plugin', function (options) {
    return function (root) {
        root.walkComments((comment) => {
            skipConditionalCompilation(comment, options);
            comment.remove();
        });
    };
});
function skipConditionalCompilation(comment, options) {
    if ((0, comment_1.isIfDef)(comment.text)) {
        const conditional = (0, comment_1.defCondition)(comment.text).toLowerCase();
        if (conditional !== options.platform) {
            const removeItems = [];
            let startRemove = false;
            for (const el of comment.parent.nodes) {
                if (el.type === 'comment') {
                    if (el === comment) {
                        startRemove = true;
                        continue;
                    }
                    else if ((0, comment_1.isEndIf)(el.text)) {
                        startRemove = false;
                        break;
                    }
                }
                else {
                    if (startRemove) {
                        removeItems.push(el);
                    }
                }
            }
            // 删除 不符合条件编译 的 节点
            removeItems.forEach((el) => el.remove());
        }
    }
}
//# sourceMappingURL=index.js.map