"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadableComponents = void 0;
const utils_1 = require("@morjs/utils");
const path_1 = __importDefault(require("path"));
const constants_1 = require("../../../../constants");
const file_utils_1 = require("../../../utils/file-utils");
const index_1 = require("../../../utils/index");
const option_1 = require("../../option");
const types_1 = require("../ast/types");
const babel_helper_1 = require("../babel-helper");
const traverse_1 = __importDefault(require("../traverse"));
const config_default_1 = __importDefault(require("./config-default"));
const context_1 = __importDefault(require("./context"));
const elements_1 = __importDefault(require("./elements"));
const CommentNode_1 = require("./elements/CommentNode");
const utils_2 = require("./utils");
function isComponent(options) {
    return options.config && options.config.component;
}
function clearLoadableComponents() {
    exports.loadableComponents = [];
}
exports.loadableComponents = [];
function default_1(document, options) {
    const protectIdentifyNames = [...config_default_1.default.GlobalIdentifyNames];
    document.imports
        .filter((i) => i.name)
        .forEach((i) => {
        const valueNames = (0, babel_helper_1.takeObjectExpresionValueNames)(i.name);
        if (valueNames && valueNames.length > 0) {
            protectIdentifyNames.push(...valueNames);
        }
        else {
            protectIdentifyNames.push(i.name);
        }
    });
    const doc = {
        ...document,
        protectIdentifyNames,
        isComplexComponents: false
    };
    /**
     * NOTE: 先处理组件名称的引用
     */
    generateComponentImport(doc, options);
    const ast = utils_1.babelTypes.program([], [], 'module');
    // 添加 react  引用
    ast.body.push(utils_1.babelTypes.importDeclaration([utils_1.babelTypes.importDefaultSpecifier(utils_1.babelTypes.identifier('React'))], utils_1.babelTypes.stringLiteral('react')));
    // 添加 tiga runtime 引用
    ast.body.push(utils_1.babelTypes.importDeclaration([utils_1.babelTypes.importDefaultSpecifier(utils_1.babelTypes.identifier('$rm'))], utils_1.babelTypes.stringLiteral(constants_1.WEB_RUNTIMES.runtime)));
    // 添加 import
    doc.imports
        .filter((i) => i.name && i.from)
        .forEach((i) => {
        ast.body.push(utils_1.babelTypes.importDeclaration([utils_1.babelTypes.importDefaultSpecifier(utils_1.babelTypes.identifier(i.name))], utils_1.babelTypes.stringLiteral(i.from)));
    });
    // NOTE: 暂时去掉这个逻辑, web 组件需要有一定的运行环境
    // 如果是 cli 编译，那么自动引入components库
    // if (options.isAtomicMode) {
    //   ast.body.splice(
    //     0,
    //     0,
    //     t.importDeclaration([], t.stringLiteral(WEB_RUNTIMES.components))
    //   )
    // }
    // 页面的处理逻辑
    if (!isComponent(options)) {
        // NOTE: 一定要确保样式的引用是在最后的引入
        // 2. 添加css 样式文件
        if (options.hasAppStyle) {
            const importStyle = `@/${(0, file_utils_1.getAppCssResourceName)(options)}`;
            // 只有page 才会引入 app.acss
            if (options.isAtomicMode) {
                ast.body.push(utils_1.babelTypes.importDeclaration([], utils_1.babelTypes.stringLiteral(importStyle.endsWith('.css') ? importStyle : `${importStyle}.css`)));
            }
            else {
                ast.body.push(utils_1.babelTypes.importDeclaration([], utils_1.babelTypes.stringLiteral(importStyle)));
            }
        }
    }
    // 添加当前文件对应的样式文件引入
    if (options.hasStyleFile) {
        const importStyle = (0, file_utils_1.getCssResourcePath)(options);
        if (options.isAtomicMode) {
            ast.body.push(utils_1.babelTypes.importDeclaration([], utils_1.babelTypes.stringLiteral(importStyle.endsWith('.css') ? importStyle : `${importStyle}.css`)));
        }
        else {
            ast.body.push(utils_1.babelTypes.importDeclaration([], utils_1.babelTypes.stringLiteral(importStyle)));
        }
    }
    addLineFeed(ast.body);
    clearLoadableComponents();
    // NOTE: 添加模板
    ast.body.push(...buildTemplates(doc, options));
    // 添加defualt 导出
    addLineFeed(ast.body);
    ast.body.push(utils_1.babelTypes.exportNamedDeclaration(buildFragment(options, doc.fragment, doc, 'defaultRender', !isComponent(options))));
    utils_1.babelTypes.addComment(ast.body[ast.body.length - 1], 'leading', '添加渲染函数', true);
    ast.body.push(utils_1.babelTypes.exportNamedDeclaration(utils_1.babelTypes.variableDeclaration('const', [
        utils_1.babelTypes.variableDeclarator(utils_1.babelTypes.identifier('isComplexComponents'), utils_1.babelTypes.booleanLiteral(doc.isComplexComponents))
    ]), []));
    // 组件按需加载
    exports.loadableComponents.forEach((key) => {
        const componentPath = (0, index_1.getExternalComponent)(options, key);
        if (componentPath) {
            ast.body.splice(3, 0, utils_1.babelTypes.importDeclaration([utils_1.babelTypes.importDefaultSpecifier(utils_1.babelTypes.identifier((0, utils_2.getComponentName)(key)))], utils_1.babelTypes.stringLiteral(componentPath)));
        }
        else {
            ast.body.splice(3, 0, utils_1.babelTypes.importDeclaration([], utils_1.babelTypes.stringLiteral(`${constants_1.WEB_RUNTIMES.components}/src/loadable/${key}`)));
        }
    });
    const result = utils_1.babelGenerator.default(ast);
    if (options.isAtomicMode && !options.unitTest) {
        return utils_1.babelCore.transformSync(result.code, {
            presets: [require((0, utils_1.resolveDependency)('@babel/preset-react'))]
        }).code;
    }
    return result.code;
}
exports.default = default_1;
/**
 * 编一个代码片段(runtime 函数)
 * @param fragment
 */
function buildFragment(options, fragment, doc, functionName, isPage = false) {
    const context = new context_1.default(options);
    let body = fragment.body;
    if (isPage) {
        body = [(0, types_1.elementNode)('tiga-page', '', [], body)];
    }
    let ast;
    if (body.length === 0) {
        ast = utils_1.babelTypes.booleanLiteral(false);
    }
    else if (body.length === 1) {
        ast = (0, elements_1.default)(body[0], context);
        const { type, name } = body[0];
        if (type === 'SlotNode' ||
            type === 'BlockNode' ||
            !(0, option_1.isGlobalComponent)(name, options)) {
            doc.isComplexComponents = true;
        }
    }
    else {
        const block = (0, types_1.blockNode)(body);
        ast = (0, elements_1.default)(block, context);
        doc.isComplexComponents = true;
    }
    if (utils_1.babelTypes.isJSXExpressionContainer(ast)) {
        ast = ast.expression;
        doc.isComplexComponents = true;
    }
    else if (utils_1.babelTypes.isJSXText(ast)) {
        ast = utils_1.babelTypes.stringLiteral(ast.value);
    }
    const vars = [];
    context.dataBindingVars.forEach((v) => {
        if (config_default_1.default.DSLProtectIdenfifyName.indexOf(v) > -1) {
            throw new Error(`变量名 ${v} 为保留变量名，请不要在AXML中使用`);
        }
        if (doc.protectIdentifyNames.indexOf(v) === -1) {
            vars.push(v);
        }
    });
    const statements = [];
    if (vars.length > 0) {
        statements.push(utils_1.babelTypes.variableDeclaration('const', [
            utils_1.babelTypes.variableDeclarator(utils_1.babelTypes.objectPattern(vars.map((item) => utils_1.babelTypes.objectProperty(utils_1.babelTypes.identifier(item), utils_1.babelTypes.identifier(item), false, true))), utils_1.babelTypes.identifier('$data'))
        ]));
    }
    statements.push(utils_1.babelTypes.returnStatement(ast));
    return utils_1.babelTypes.functionDeclaration(functionName ? utils_1.babelTypes.identifier(functionName) : null, [utils_1.babelTypes.identifier('$data')], utils_1.babelTypes.blockStatement(statements));
}
/**
 * 编译模板
 * @param doc
 */
function buildTemplates(doc, options) {
    const body = [];
    // 添加 模板的导出
    body.push(utils_1.babelTypes.exportNamedDeclaration(utils_1.babelTypes.variableDeclaration('const', [
        utils_1.babelTypes.variableDeclarator(utils_1.babelTypes.identifier('$templates'), utils_1.babelTypes.objectExpression([]))
    ]), []));
    // 过滤需要引入的模版
    const templateImports = doc.imports.filter((item) => item.src);
    // 如果没有引入模版, 则不执行后续代码生成
    if (!(templateImports === null || templateImports === void 0 ? void 0 : templateImports.length) && !doc.templates.length)
        return body;
    // 创建模板管理器
    addLineFeed(body);
    body.push(utils_1.babelTypes.variableDeclaration('const', [
        utils_1.babelTypes.variableDeclarator(utils_1.babelTypes.identifier('$tm'), utils_1.babelTypes.callExpression(utils_1.babelTypes.memberExpression(utils_1.babelTypes.identifier('$rm'), utils_1.babelTypes.identifier('createTemplateManager')), []))
    ]));
    // 添加注释
    utils_1.babelTypes.addComment(body[body.length - 1], 'leading', '创建模板管理器', true);
    // runtime api
    body.push(utils_1.babelTypes.variableDeclaration('const', [
        utils_1.babelTypes.variableDeclarator(utils_1.babelTypes.identifier('runtime'), utils_1.babelTypes.callExpression(utils_1.babelTypes.memberExpression(utils_1.babelTypes.identifier('$rm'), utils_1.babelTypes.identifier('axmlApi')), [utils_1.babelTypes.identifier('$tm')]))
    ]));
    // 添加注释
    utils_1.babelTypes.addComment(body[body.length - 1], 'leading', '创建 runtime api', true);
    addLineFeed(body);
    // 创建模板
    if (doc.templates.length > 0) {
        doc.templates.forEach((template) => {
            const tName = `template${(0, utils_2.hashTemplateName)(template.name)}`;
            body.push(buildFragment(options, template.fragment, doc, tName));
            utils_1.babelTypes.addComment(body[body.length - 1], 'leading', '创建模板:' + tName, true);
            body.push(utils_1.babelTypes.expressionStatement(utils_1.babelTypes.assignmentExpression('=', utils_1.babelTypes.memberExpression(utils_1.babelTypes.identifier('$templates'), utils_1.babelTypes.stringLiteral(tName), true), utils_1.babelTypes.identifier(tName))));
        });
        // 将当前文件定义的模板加入模板管理器
        body.push(utils_1.babelTypes.expressionStatement(utils_1.babelTypes.callExpression(utils_1.babelTypes.memberExpression(utils_1.babelTypes.identifier('$tm'), utils_1.babelTypes.identifier('addAll')), [utils_1.babelTypes.identifier('$templates')])));
        utils_1.babelTypes.addComment(body[body.length - 1], 'leading', '将当前文件定义的模板加入模板管理器', true);
    }
    // 引入其他文件中的模板
    templateImports.forEach((item) => {
        body.push(utils_1.babelTypes.expressionStatement(utils_1.babelTypes.callExpression(utils_1.babelTypes.memberExpression(utils_1.babelTypes.identifier('$tm'), utils_1.babelTypes.identifier('addAll')), [
            utils_1.babelTypes.memberExpression(utils_1.babelTypes.callExpression(utils_1.babelTypes.identifier('require'), [
                utils_1.babelTypes.stringLiteral(item.src)
            ]), utils_1.babelTypes.identifier('$templates'))
        ])));
        utils_1.babelTypes.addComment(body[body.length - 1], 'leading', '引入其他文件的模板：' + item.src, true);
    });
    return body;
}
/**
 * 添加换行
 * @param body
 */
function addLineFeed(body) {
    body.push(utils_1.babelTypes.jsxText('\r'));
}
/**
 * 生成组件引用的代码
 */
function generateComponentImport(doc, options) {
    // NOTE: 第一步，先把 app.json 中的组件路径转换成当前编译文件的相对路径
    // 先读取app 中的 usingComponents
    const appUsingComponents = { ...options.appConfig.usingComponents };
    // 转换app.json 中路径
    Object.keys(appUsingComponents).forEach((key) => {
        if (key.indexOf('.') >= 0) {
            throw new Error('usingComponents 的 key，不支持 "." \n' + options.resourcePath);
        }
        const value = appUsingComponents[key];
        if (value.startsWith('.')) {
            //將相對路徑轉換成絕對路徑
            const abPath = path_1.default.resolve(options.rootPath, value);
            appUsingComponents[key] = abPath.replace(options.rootPath, '');
        }
    });
    // Step2:  合并 UsingComponents
    const UsingComponents = Object.assign(appUsingComponents, options.config && options.config.usingComponents);
    for (const key in UsingComponents) {
        const value = UsingComponents[key];
        if (value[0] === '/') {
            // 将项目的绝对路径，转换成，实际的本地绝对路径
            UsingComponents[key] = `@${value}`;
        }
    }
    const usedComponents = {};
    // 引入到axml中的文件可以直接忽略key
    const importKeys = doc.imports.map((i) => i.name).filter((i) => i);
    // 先进行ast 遍历。
    (0, traverse_1.default)(doc, {
        ElementNode(_) {
            const node = _.node;
            // 针对节点名称，进行节点转换。
            if (node.name in UsingComponents &&
                importKeys.indexOf(node.name) === -1) {
                const name = (0, utils_2.convertComponentName)(node.name);
                usedComponents[name] = UsingComponents[node.name];
                node.name = name;
            }
        },
        // NOTE:条件编译，剔除不需要编译的节点
        CommentNode(_) {
            if ((0, types_1.isFragmentNode)(_.parent)) {
                const parent = _.parent;
                parent.body = (0, CommentNode_1.skipConditionalCompilation)(_.node, parent.body, options);
            }
            else {
                const children = _.parent['children'];
                if (children && children instanceof Array) {
                    _.parent['children'] = (0, CommentNode_1.skipConditionalCompilation)(_.node, children, options);
                }
            }
        }
    });
    Object.keys(usedComponents).forEach((key) => {
        doc.imports.push((0, types_1.importNode)(null, key, usedComponents[key]));
    });
}
//# sourceMappingURL=index.js.map