"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addBracketToIdentifier = exports.getOptionalChainExpression = exports.getOptionalChainCode = exports.uuid = exports.getComponentName = exports.convertComponentName = exports.hashTemplateName = void 0;
const utils_1 = require("@morjs/utils");
const HashChars = 'ABCDEFGHJKMNPQRSTWXYZabcdefhijkmnprstwxyz2345678oOLl9gqVvUuI1';
// 对模板名称进行编码
// NOTE: 这里的代码必须跟runtime中的代码是一致的
function hashTemplateName(name) {
    return name
        .split('')
        .map((c) => (HashChars.indexOf(c) >= 0 ? c : '$'))
        .join('');
}
exports.hashTemplateName = hashTemplateName;
// 将引用的组件名称，转换成合乎react规范的名称
function convertComponentName(name) {
    // 1. 替换 - 为 _
    name = name.replace(/-/g, '_');
    // 2. 添加一个大写的前缀
    name = `RT_${name}`; // RT表示runtime
    return name;
}
exports.convertComponentName = convertComponentName;
function getComponentName(name) {
    if (name.includes('-')) {
        const words = name.split('-');
        if (words === null || words === void 0 ? void 0 : words.length) {
            const nodeName = words
                .map((word) => word.charAt(0).toUpperCase() + word.slice(1))
                .join('');
            return nodeName;
        }
    }
    return name.charAt(0).toUpperCase() + name.slice(1);
}
exports.getComponentName = getComponentName;
function uuid() {
    return 'xxxxxxxxxxxx4xxxyxxxxxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
        const r = (Math.random() * 16) | 0, v = c === 'x' ? r : (r & 0x3) | 0x8;
        return v.toString(16);
    });
}
exports.uuid = uuid;
function getOptionalChainCode(code) {
    // 处理 ?.[]?.
    const result = code
        .replace(/(\w\]*)(?=\[)/g, (path) => {
        if (/\w\]*/.test(path)) {
            return `${path}?.`;
        }
        return path;
    })
        .replace(/(?<=\])(\.)/g, (path) => {
        if (/\./.test(path)) {
            return `?${path}`;
        }
        return path;
    });
    const source = utils_1.babelCore.transformSync(result, {
        plugins: [
            {
                visitor: {
                    MemberExpression: {
                        enter(path) {
                            if (path.node.property.type === 'Identifier') {
                                path.replaceWith(utils_1.babelTypes.optionalMemberExpression(path.node.object, path.node.property, false, true));
                            }
                        }
                    }
                }
            }
        ]
    }).code;
    return source.substr(0, source.length - 1);
}
exports.getOptionalChainCode = getOptionalChainCode;
function getOptionalChainExpression(expression) {
    let source = expression;
    // ast
    if (expression.type) {
        if (expression.type === 'MemberExpression' ||
            expression.type === 'ConditionalExpression' ||
            expression.type === 'BinaryExpression' ||
            expression.type === 'LogicalExpression') {
            source = utils_1.babelGenerator.default(expression).code;
        }
        else {
            return expression;
        }
    }
    return utils_1.babelTypes.identifier(`${getOptionalChainCode(source)}`);
}
exports.getOptionalChainExpression = getOptionalChainExpression;
function addBracketToIdentifier(node) {
    if (!node)
        return node;
    if (node.type !== 'Identifier')
        return node;
    const value = `(${node.name})`;
    node.name = value;
    return node;
}
exports.addBracketToIdentifier = addBracketToIdentifier;
//# sourceMappingURL=utils.js.map