"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@morjs/utils");
const constants_1 = require("../../../constants");
const comment_1 = require("../../utils/comment");
const file_utils_1 = require("../../utils/file-utils");
const component_1 = __importDefault(require("./component"));
function default_1(content, options) {
    var _a, _b;
    const { isAtomicMode, templateFilePath, hasAppConfig, appConfigPath, resourcePath, configFilePath } = options;
    const config = options.config || {};
    if (templateFilePath) {
        content = `
    ${isAtomicMode ? 'import "' + constants_1.WEB_RUNTIMES.api + '"\r\n' : ''}import $rm from "${constants_1.WEB_RUNTIMES.runtime}";
    const $componentInfo$ = require("${(0, file_utils_1.getAxmlResourcePath)(options)}");
    ${content}
    export default $rm.${config.component ? 'Component' : 'Page'}($componentInfo$, $rm.mergeConfig(${hasAppConfig
            ? `require('${(0, utils_1.getRelativePath)(resourcePath, appConfigPath)}')`
            : '{}'}, ${Object.keys(config).length === 0 ? '{}' : `require('${configFilePath}')`}))
    `;
    }
    const plugins = [[component_1.default, { options }]];
    if ((_b = (_a = options === null || options === void 0 ? void 0 : options.compilerPlugins) === null || _a === void 0 ? void 0 : _a.js) === null || _b === void 0 ? void 0 : _b.length) {
        options.compilerPlugins.js.forEach((plugin) => {
            plugins.push([plugin, { options }]);
        });
    }
    const jsString = transformFromCode(content, plugins, options);
    return jsString;
}
exports.default = default_1;
function transformFromCode(code, plugins, options) {
    let ast = utils_1.babelCore.parse(code, {
    // plugins: [require(resolveDependency('@babel/plugin-transform-react-jsx')), require(resolveDependency('@babel/plugin-proposal-class-properties'))]
    });
    const newCode = skipConditionalCompilation(code, ast, options);
    if (newCode.length !== code.length) {
        code = newCode;
        //  重新生成ast
        ast = utils_1.babelCore.parse(code);
    }
    const babelConfig = {
        compact: false,
        generatorOpts: { comments: false },
        ast: true,
        plugins
    };
    const tramformResult = utils_1.babelCore.transformFromAstSync(ast, code, babelConfig);
    const result = utils_1.babelGenerator.default(tramformResult.ast);
    return result.code;
}
/**
 * 过滤条件编译的代码
 * @param code
 */
function skipConditionalCompilation(code, ast, options) {
    const removePostions = [];
    let startDefComment;
    if (ast.comments && ast.comments.length > 0) {
        for (const comment of ast.comments) {
            if (comment.type === 'CommentBlock') {
                if ((0, comment_1.isIfDef)(comment.value)) {
                    const conditional = (0, comment_1.defCondition)(comment.value).toLowerCase();
                    if (conditional !== options.platform) {
                        startDefComment = comment;
                    }
                }
                else if ((0, comment_1.isEndIf)(comment.value)) {
                    if (startDefComment) {
                        removePostions.push({
                            start: startDefComment.start,
                            end: comment.end
                        });
                        startDefComment = null;
                    }
                }
            }
        }
    }
    if (removePostions.length > 0) {
        let removeLength = 0;
        removePostions.forEach((item) => {
            code =
                code.substring(0, item.start - removeLength) +
                    code.substring(item.end - removeLength);
            removeLength = item.end - item.start;
        });
    }
    return code;
}
//# sourceMappingURL=index.js.map