import { zod as z } from '@morjs/utils';
/**
 * 以下为 mor 编译插件需要
 */
export declare const target = "web";
export declare const targetDescription = "Web \u5E94\u7528";
export declare const globalObject = "my";
export declare const resolveMainFields: string[];
export declare const defaultConditionalFileExt = ".web";
export declare const defaultOutputDir = "dist/web";
export declare const projectConfigFiles: any[];
export declare const templateSingleTagClosingType = "tag";
/**
 * 静态资源支持
 */
export declare const ASSETS_REGEXP: RegExp;
/**
 * 当前 npm 所包含的 node_modules 目录
 */
export declare const CURRENT_NODE_MODUELS: string;
export declare const RUNTIME_NPM_NAME = "@morjs/runtime-web";
export declare const WEB_RUNTIMES: {
    runtime: string;
    api: string;
    components: string;
    router: string;
    config: string;
};
export declare const WEB_COMPILER_LOADERS: {
    template: string;
    script: string;
    style: string;
};
/**
 * 生成虚拟的 entry 根目录
 */
export declare function generateEntryRoot(srcPath: string): string;
/**
 * 获取 webpack 文件名称配置
 */
export declare function fileNameConfig(mode: string): "[name].[contenthash:6]" | "[name]";
/**
 * Web 相关用户配置
 */
export declare const UserConfigSchema: z.ZodObject<{
    /**
     * web 编译相关配置
     */
    web: z.ZodDefault<z.ZodObject<{
        globalComponentsConfig: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodString>>;
        needRpx: z.ZodOptional<z.ZodBoolean>;
        styleScope: z.ZodOptional<z.ZodBoolean>;
        rpxRootValue: z.ZodOptional<z.ZodNumber>;
        usePx: z.ZodOptional<z.ZodObject<{
            ratio: z.ZodNumber;
        }, "strip", z.ZodTypeAny, {
            ratio?: number;
        }, {
            ratio?: number;
        }>>;
        extensions: z.ZodOptional<z.ZodArray<z.ZodUnion<[z.ZodString, z.ZodTuple<[z.ZodString, z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodAny>>], null>]>, "many">>;
        responsiveRootFontSize: z.ZodOptional<z.ZodNumber>;
        showBack: z.ZodDefault<z.ZodUnion<[z.ZodUnion<[z.ZodBoolean, z.ZodArray<z.ZodString, "many">]>, z.ZodFunction<z.ZodTuple<[z.ZodString], z.ZodUnknown>, z.ZodBoolean>]>>;
        showHeader: z.ZodDefault<z.ZodUnion<[z.ZodUnion<[z.ZodBoolean, z.ZodArray<z.ZodString, "many">]>, z.ZodFunction<z.ZodTuple<[z.ZodString], z.ZodUnknown>, z.ZodBoolean>]>>;
        publicPath: z.ZodDefault<z.ZodString>;
        devServer: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodAny>>;
        htmlOptions: z.ZodOptional<z.ZodUnion<[z.ZodArray<z.ZodRecord<z.ZodString, z.ZodAny>, "many">, z.ZodRecord<z.ZodString, z.ZodAny>]>>;
        miniCssExtractOptions: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodAny>>;
        inlineAssetLimit: z.ZodDefault<z.ZodNumber>;
        autoGenerateEntries: z.ZodDefault<z.ZodBoolean>;
        entry: z.ZodOptional<z.ZodAny>;
        emitIntermediateAssets: z.ZodDefault<z.ZodBoolean>;
        appConfig: z.ZodDefault<z.ZodObject<{
            /**
             * 用于传递业务对某个组件的特殊配置, 比如 map 组件传递 key
             */
            components: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodAny>>;
            /**
             * 用于自定义 URL 中 query 部分的 tabBarKey 的实际名称，该 key 作用于 URL 的
             * query，并将拿到到值，用于获取 app.json 中的 tabbar 内容，以此实现通过 URL 参数
             * 切换不同 tabbar 的功能
             */
            customTabBarKey: z.ZodDefault<z.ZodString>;
            /**
             * 用于配置是否允许覆盖全局对象中的方法
             */
            apiNoConflict: z.ZodOptional<z.ZodBoolean>;
            pageHeaderConfig: z.ZodOptional<z.ZodObject<{
                showHeader: z.ZodOptional<z.ZodBoolean>;
                showBack: z.ZodOptional<z.ZodBoolean>;
            }, "strip", z.ZodTypeAny, {
                showHeader?: boolean;
                showBack?: boolean;
            }, {
                showHeader?: boolean;
                showBack?: boolean;
            }>>;
        }, "strip", z.ZodTypeAny, {
            components?: Record<string, any>;
            customTabBarKey?: string;
            apiNoConflict?: boolean;
            pageHeaderConfig?: {
                showHeader?: boolean;
                showBack?: boolean;
            };
        }, {
            components?: Record<string, any>;
            customTabBarKey?: string;
            apiNoConflict?: boolean;
            pageHeaderConfig?: {
                showHeader?: boolean;
                showBack?: boolean;
            };
        }>>;
    }, "strip", z.ZodTypeAny, {
        globalComponentsConfig?: Record<string, string>;
        needRpx?: boolean;
        styleScope?: boolean;
        rpxRootValue?: number;
        usePx?: {
            ratio?: number;
        };
        extensions?: (string | [string, Record<string, any>, ...unknown[]])[];
        responsiveRootFontSize?: number;
        showBack?: boolean | string[] | ((args_0: string, ...args_1: unknown[]) => boolean);
        showHeader?: boolean | string[] | ((args_0: string, ...args_1: unknown[]) => boolean);
        publicPath?: string;
        devServer?: Record<string, any>;
        htmlOptions?: Record<string, any> | Record<string, any>[];
        miniCssExtractOptions?: Record<string, any>;
        inlineAssetLimit?: number;
        autoGenerateEntries?: boolean;
        entry?: any;
        emitIntermediateAssets?: boolean;
        appConfig?: {
            components?: Record<string, any>;
            customTabBarKey?: string;
            apiNoConflict?: boolean;
            pageHeaderConfig?: {
                showHeader?: boolean;
                showBack?: boolean;
            };
        };
    }, {
        globalComponentsConfig?: Record<string, string>;
        needRpx?: boolean;
        styleScope?: boolean;
        rpxRootValue?: number;
        usePx?: {
            ratio?: number;
        };
        extensions?: (string | [string, Record<string, any>, ...unknown[]])[];
        responsiveRootFontSize?: number;
        showBack?: boolean | string[] | ((args_0: string, ...args_1: unknown[]) => boolean);
        showHeader?: boolean | string[] | ((args_0: string, ...args_1: unknown[]) => boolean);
        publicPath?: string;
        devServer?: Record<string, any>;
        htmlOptions?: Record<string, any> | Record<string, any>[];
        miniCssExtractOptions?: Record<string, any>;
        inlineAssetLimit?: number;
        autoGenerateEntries?: boolean;
        entry?: any;
        emitIntermediateAssets?: boolean;
        appConfig?: {
            components?: Record<string, any>;
            customTabBarKey?: string;
            apiNoConflict?: boolean;
            pageHeaderConfig?: {
                showHeader?: boolean;
                showBack?: boolean;
            };
        };
    }>>;
}, "strip", z.ZodTypeAny, {
    web?: {
        globalComponentsConfig?: Record<string, string>;
        needRpx?: boolean;
        styleScope?: boolean;
        rpxRootValue?: number;
        usePx?: {
            ratio?: number;
        };
        extensions?: (string | [string, Record<string, any>, ...unknown[]])[];
        responsiveRootFontSize?: number;
        showBack?: boolean | string[] | ((args_0: string, ...args_1: unknown[]) => boolean);
        showHeader?: boolean | string[] | ((args_0: string, ...args_1: unknown[]) => boolean);
        publicPath?: string;
        devServer?: Record<string, any>;
        htmlOptions?: Record<string, any> | Record<string, any>[];
        miniCssExtractOptions?: Record<string, any>;
        inlineAssetLimit?: number;
        autoGenerateEntries?: boolean;
        entry?: any;
        emitIntermediateAssets?: boolean;
        appConfig?: {
            components?: Record<string, any>;
            customTabBarKey?: string;
            apiNoConflict?: boolean;
            pageHeaderConfig?: {
                showHeader?: boolean;
                showBack?: boolean;
            };
        };
    };
}, {
    web?: {
        globalComponentsConfig?: Record<string, string>;
        needRpx?: boolean;
        styleScope?: boolean;
        rpxRootValue?: number;
        usePx?: {
            ratio?: number;
        };
        extensions?: (string | [string, Record<string, any>, ...unknown[]])[];
        responsiveRootFontSize?: number;
        showBack?: boolean | string[] | ((args_0: string, ...args_1: unknown[]) => boolean);
        showHeader?: boolean | string[] | ((args_0: string, ...args_1: unknown[]) => boolean);
        publicPath?: string;
        devServer?: Record<string, any>;
        htmlOptions?: Record<string, any> | Record<string, any>[];
        miniCssExtractOptions?: Record<string, any>;
        inlineAssetLimit?: number;
        autoGenerateEntries?: boolean;
        entry?: any;
        emitIntermediateAssets?: boolean;
        appConfig?: {
            components?: Record<string, any>;
            customTabBarKey?: string;
            apiNoConflict?: boolean;
            pageHeaderConfig?: {
                showHeader?: boolean;
                showBack?: boolean;
            };
        };
    };
}>;
/**
 * Web 编译配置类型
 */
export declare type WebCompilerUserConfig = z.infer<typeof UserConfigSchema>;
