"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EmitIntermediateAssetsPlugin = void 0;
const utils_1 = require("@morjs/utils");
const path_1 = __importDefault(require("path"));
const constants_1 = require("../constants");
/**
 * Web 相关通用配置支持
 */
class EmitIntermediateAssetsPlugin {
    constructor() {
        this.name = 'MorWebEmitIntermediateAssetsPlugin';
    }
    apply(runner) {
        runner.hooks.cli.tap(this.name, function (cli) {
            cli
                .command('compile')
                .option('--emit-web-intermediate-assets', '生成 web 转端中间产物 (方便调试)');
        });
        // 命令行选项覆盖用户配置
        runner.hooks.modifyUserConfig.tap(this.name, function (userConfig, command) {
            var _a;
            if ((_a = command.options) === null || _a === void 0 ? void 0 : _a.emitWebIntermediateAssets) {
                if (userConfig.target === constants_1.target) {
                    userConfig.web = userConfig.web || {};
                    userConfig.web.emitIntermediateAssets =
                        command.options.emitWebIntermediateAssets;
                }
                else {
                    utils_1.logger.warnOnce(`仅在 target 为 web 时支持 --emit-web-intermediate-assets 选项`);
                }
            }
            return userConfig;
        });
        runner.hooks.userConfigValidated.tapPromise(this.name, async (userConfig) => {
            var _a;
            if ((_a = userConfig.web) === null || _a === void 0 ? void 0 : _a.emitIntermediateAssets) {
                const intermediateAssetsDir = path_1.default.join(runner.config.getTempDir(), 'compiler', 'intermediate-assets', userConfig.name || userConfig.target);
                // 自动清空中间产物目录
                if (await utils_1.fsExtra.pathExists(intermediateAssetsDir)) {
                    utils_1.fsExtra.emptyDir(intermediateAssetsDir);
                }
                const cwd = runner.getCwd();
                utils_1.logger.info(`已开启 web 转端中间产物输出: ${path_1.default.relative(cwd, intermediateAssetsDir)}`);
                // 介入到 postprocessor 阶段用于输出中间产物
                runner.hooks.postprocessorParser.tapPromise({
                    name: this.name,
                    // 确保最终产物拿到的是最后处理完成的内容
                    stage: Number.POSITIVE_INFINITY
                }, async function (source, options) {
                    var _a;
                    // 优先使用 entry 名称作为中间产物文件路径
                    let filePath = ((_a = options === null || options === void 0 ? void 0 : options.fileInfo) === null || _a === void 0 ? void 0 : _a.entryName) ||
                        path_1.default.relative(cwd, options.fileInfo.path);
                    switch (options === null || options === void 0 ? void 0 : options.fileInfo.entryFileType) {
                        case utils_1.EntryFileType.sjs:
                        case utils_1.EntryFileType.template:
                            filePath = filePath + '.js';
                            break;
                    }
                    await utils_1.fsExtra.outputFile(path_1.default.join(intermediateAssetsDir, path_1.default.resolve('/', filePath)), source);
                    return source;
                });
            }
        });
    }
}
exports.EmitIntermediateAssetsPlugin = EmitIntermediateAssetsPlugin;
//# sourceMappingURL=emitIntermediateAssetsPlugin.js.map