"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExtractOrInjectCssPlugin = void 0;
const utils_1 = require("@morjs/utils");
const constants_1 = require("../constants");
/**
 * CSS 提取或注入配置
 */
class ExtractOrInjectCssPlugin {
    constructor(wrapper) {
        this.wrapper = wrapper;
        this.name = 'MorWebExtractOrInjectCssPlugin';
    }
    apply(runner) {
        const { web = {}, mode } = (runner.userConfig ||
            {});
        this.setupCssSupport(web, (0, constants_1.fileNameConfig)(mode));
    }
    /**
     * 配置 css 相关支持
     */
    setupCssSupport(web, fileName) {
        const chain = this.wrapper.chain;
        const MiniCssExtractPlugin = require((0, utils_1.resolveDependency)('mini-css-extract-plugin'));
        let extractOrStyleLoader;
        if (web.miniCssExtractOptions) {
            extractOrStyleLoader = MiniCssExtractPlugin.loader;
            chain.plugin('mini-css-extract-plugin').use(MiniCssExtractPlugin, [
                {
                    filename: `${fileName}.css`,
                    chunkFilename: `${fileName}.css`,
                    ...web.miniCssExtractOptions
                }
            ]);
        }
        else {
            extractOrStyleLoader = (0, utils_1.resolveDependency)('style-loader');
        }
        // importLoaders 代表通过 @import 引入的文件经过几个 loader
        // 这里设置一个较大的数字，来确保所有样式 loader 都可以被使用到
        const cssLoaderOptions = { importLoaders: 100 };
        // 样式处理
        // prettier-ignore
        chain.module
            .rule('style')
            .use('css')
            .loader((0, utils_1.resolveDependency)('css-loader'))
            .options(cssLoaderOptions)
            .before('postprocess')
            .end()
            .use('extractOrStyle')
            .loader(extractOrStyleLoader)
            .before('css')
            .end();
        // 样式处理
        // prettier-ignore
        chain.module
            .rule('less')
            .use('css')
            .loader((0, utils_1.resolveDependency)('css-loader'))
            .options(cssLoaderOptions)
            .before('postprocess')
            .end()
            .use('extractOrStyle')
            .loader(extractOrStyleLoader)
            .before('css')
            .end();
        // 样式处理
        // prettier-ignore
        chain.module
            .rule('sass')
            .use('css')
            .loader((0, utils_1.resolveDependency)('css-loader'))
            .options(cssLoaderOptions)
            .before('postprocess')
            .end()
            .use('extractOrStyle')
            .loader(extractOrStyleLoader)
            .before('css')
            .end();
    }
}
exports.ExtractOrInjectCssPlugin = ExtractOrInjectCssPlugin;
//# sourceMappingURL=extractOrInjectCssPlugin.js.map