"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TemplateCompatiblePlugin = void 0;
const utils_1 = require("@morjs/utils");
const path_1 = __importDefault(require("path"));
const utils_2 = require("../utils");
/**
 * Template 兼容插件
 * 1. 替换 模版和 sjs 为正确的路径
 * 2. 将文件纬度条件编译的结果应用到文件本身
 */
class TemplateCompatiblePlugin {
    constructor(entryBuilder) {
        this.entryBuilder = entryBuilder;
        this.name = 'MorWebTemplateCompatiblePlugin';
    }
    apply(runner) {
        // 替换 模版和 sjs 为正确的路径
        // 将文件纬度条件编译的结果应用到文件本身
        // NOTE: sjs 引用路径需要支持替换
        runner.hooks.templateParser.tap(this.name, (tree, options) => {
            const entryBuilder = this.entryBuilder;
            return tree.walk((node) => {
                if (node.tag === 'import' || node.tag === 'include') {
                    if (node.attrs &&
                        node.attrs.src &&
                        typeof node.attrs.src === 'string') {
                        const importPath = node.attrs.src;
                        const realPath = entryBuilder.getFullPathOfReferenceFile(options.fileInfo.path, importPath);
                        if (realPath) {
                            let relativePath = path_1.default.relative(path_1.default.dirname(options.fileInfo.path), realPath);
                            relativePath = (0, utils_1.slash)(relativePath.startsWith('.')
                                ? relativePath
                                : './' + relativePath);
                            node.attrs.src = relativePath;
                        }
                        else {
                            node.attrs.src = importPath;
                        }
                    }
                    else {
                        utils_1.logger.warn('import/include 标签需要设置 src 属性' +
                            `文件路径: ${options.fileInfo.path}`);
                    }
                }
                else if (node.tag === 'image') {
                    if (node.attrs &&
                        node.attrs.src &&
                        typeof node.attrs.src === 'string') {
                        node.attrs.src = (0, utils_2.getRelativePath)(node.attrs.src);
                    }
                }
                return node;
            });
        });
    }
}
exports.TemplateCompatiblePlugin = TemplateCompatiblePlugin;
//# sourceMappingURL=templateCompatiblePlugin.js.map